@@ -6,6 +6,8 @@ from .. import exceptions
 from ..charge_object import Charge
 from ..error_result import ErrorResult
 from ..payment_object import Payment
+from ..plan_object import Plan
+from ..subscription_object import Subscription
 from ..successful_result import SuccessfulResult
 from ..utils import camelize, underscoreize
 
@@ -26,6 +28,10 @@ regex_charges_split = "^\/charges\/(.*)\/split\/$"
 regex_payments = "^\/payments\/$"
 regex_payments_capture = "^\/payments\/(.*)\/capture\/$"
 regex_payments_refunds = "^\/payments\/(.*)\/refunds\/$"
+regex_plans = "^\/plans\/$"
+regex_plans_detail = "^\/plans\/(.*)\/$"
+regex_subscriptions = "^\/subscriptions\/$"
+regex_subscriptions_detail = "^\/subscriptions\/(.*)\/$"
 
 
 def data_authorization():
@@ -65,8 +71,11 @@ def validate_authorization(juno_response):
     return error(response_json)
 
 
-def get_data_charges(data):
-    if len(data["_embedded"]["charges"]) == 1:
+def get_data_charges(data, method):
+    if "_embedded" not in data:
+        return {"charges": []}
+
+    if len(data["_embedded"]["charges"]) == 1 and method == "POST":
         return {"charge": Charge(data["_embedded"]["charges"][0])}
 
     return {
@@ -81,12 +90,37 @@ def get_data_payments(data):
     return {"payments": [Payment(payment_dict) for payment_dict in data["payments"]]}
 
 
+def get_data_plans(data, method):
+    if "_embedded" not in data:
+        return {"plans": []}
+
+    if len(data["_embedded"]["plans"]) == 1 and method == "POST":
+        return {"plan": Plan(data["_embedded"]["plans"][0])}
+
+    return {"plans": [Plan(plan_dict) for plan_dict in data["_embedded"]["plans"]]}
+
+
+def get_data_subscriptions(data, method):
+    if "_embedded" not in data:
+        return {"subscriptions": []}
+
+    if len(data["_embedded"]["subscriptions"]) == 1 and method == "POST":
+        return {"subscription": Subscription(data["_embedded"]["subscriptions"][0])}
+
+    return {
+        "subscriptions": [
+            Subscription(subscription_dict)
+            for subscription_dict in data["_embedded"]["subscriptions"]
+        ]
+    }
+
+
 def success_result(method, url, data):
     response = data
     if (method == "GET" and re.search(regex_charges, url)) or (
         method == "POST" and re.search(regex_charges, url)
     ):
-        response = get_data_charges(data)
+        response = get_data_charges(data, method)
 
     elif method == "GET" and re.search(regex_charges_detail, url):
         response = {"charge": Charge(data)}
@@ -104,6 +138,22 @@ def success_result(method, url, data):
     elif method == "POST" and re.search(regex_payments_refunds, url):
         print("POST payments refunds")
 
+    elif (method == "GET" and re.search(regex_plans, url)) or (
+        method == "POST" and re.search(regex_plans, url)
+    ):
+        response = get_data_plans(data, method)
+
+    elif method == "GET" and re.search(regex_plans_detail, url):
+        response = {"plan": Plan(data)}
+
+    elif (method == "GET" and re.search(regex_subscriptions, url)) or (
+        method == "POST" and re.search(regex_subscriptions, url)
+    ):
+        response = get_data_subscriptions(data, method)
+
+    elif method == "GET" and re.search(regex_subscriptions_detail, url):
+        response = {"subscription": Subscription(data)}
+
     return SuccessfulResult(response)
 
 
@@ -112,7 +162,7 @@ def error_result(data):
 
 
 def validate_response(method, end_point, juno_response):
-    if response_json.status_code == 204:
+    if juno_response.status_code == 204:
         return success_result(
             method, f'{end_point.replace(RESOURCE_SERVER_URL, "")}/', None
         )
@@ -163,11 +213,15 @@ def request_function(method):
 
 
 def hook_requests(method, end_point, data):
-    juno_response = request_function(method)(end_point, json=camelize(data))
+    payload = {"json": camelize(data)}
+    if method == "GET":
+        payload = {"params": camelize(data)}
+
+    juno_response = request_function(method)(end_point, **payload)
 
     if juno_response.status_code in [401, 403]:
         request_authorization()
-        return request_function(method)(end_point, json=camelize(data))
+        return request_function(method)(end_point, **payload)
 
     return juno_response
 
@@ -191,9 +245,7 @@ def delete(end_point, data={}):
 
 
 def patch(end_point, data={}):
-    return validate_response(
-        "PATCH", end_point, hook_requests("PATCH", end_point, data)
-    )
+    return validate_response("PATCH", end_point, hook_requests("PATCH", end_point, data))
 
 
 def error(data):
@@ -255,4 +307,3 @@ def raise_exception_from_error_code(error_code, message=None):
         raise exceptions.JunoAlreadyRegisteredWebhookForIndicatedEvents(message)
     else:
         raise exceptions.JunoException(message)
-
